// Filename        : memory.v
// Description     : dual ported memory array

`include "defs.h"

/* -------------------------------------------------------------
 memory array
   This is a very behavioral block and cannot be synthesized into
   primitive logic circuits.  These would typically be included as
   "black-box" macros in an ASIC (application specific integrated 
   circuit design). They are often created using memory generators
 --------------------------------------------------------------- */
module memory(clk, iaddr, daddr, write, instr, data_in, data_out);
   input  clk, write;
   input  [15:0] iaddr, daddr;
   input [15:0]  data_in;
   output [15:0] instr, data_out;

   reg [15:0] 	 mem_array[0:65535];
   reg [15:0] 	 data_out;
   
   // writes to array - behaves like clocked element, but no write/read bypassing
   always @(posedge clk)
     begin
	if(write) begin
	    mem_array[daddr] <= #`tFF data_in;
	end // if (write)
     end // always @ (posedge clk)
   
   // read instruction from memory
   assign instr = mem_array[iaddr];
   
   // read data from memory
   always @(daddr or write)
     begin
	  data_out = mem_array[daddr];
     end // always @ (daddr)

endmodule
